from rich.console import Console
from rich.panel import Panel
from rich import box
from rich.columns import Columns
from rich.syntax import Syntax
from rich.emoji import Emoji
from rich.table import Table
from art import text2art
import random

#==============================================================================

class Text:
    """
    Dynamic, interchangeable text styling for the terminal using the Rich library.
    
    Attributes:
        console: A Rich Console instance for styled output.
    """
    def __init__(self):
        self.console = Console()

    def emoji(self, name: str = "thumbs_up", style: str = "bold white"):
        """
        Display an emoji using Rich.

        Args:
            name (str, optional): The name of the emoji. Defaults to "thumbs_up".
            style (str, optional): The style of the emoji. Defaults to "bold white".
        """
        emoji = Emoji(name)
        self.console.print(emoji, style=style)

    def display(self, message: str = None, style: str = "bold white", use_box: bool = False):
            """
            Displays a message on the console.

            Args:
                message (str, optional): The message to be displayed. Defaults to None.
                style (str, optional): The style of the message. Defaults to "bold white".
                use_box (bool, optional): Whether to display the message in a box. Defaults to False.
            """
            if not message:
                message = """
                No message provided.
                """

            if isinstance(message, list):
                if use_box:
                    columns = Columns([Panel(item, expand=False, box=box.ROUNDED) for item in message])
                else:
                    columns = Columns([Panel(item, expand=False) for item in message])
                self.console.print(columns, style=style)
            else:
                if use_box:
                    message = Panel(message, expand=False, box=box.ROUNDED)
                self.console.print(message, style=style)

    def code(self, code: str = None, language: str = "python", theme: str = "monokai", line_numbers: bool = True):
        """
        Style and print a block of code using Rich Syntax.

        Example:
        ```python
        from yosemite.tools.text import Text
        text = Text()
        code = '''
        def main():
            text = Text()
            text.say("Hello, Rich!")
        '''
        text.code(code, language="python", theme="monokai", line_numbers=True)
        ```

        Args:
            code (str): The code to be styled and printed.
            language (str, optional): The language of the code. Defaults to "python".
            theme (str, optional): The theme of the code. Defaults to "monokai".
            line_numbers (bool, optional): Whether to display line numbers. Defaults to True.
        """
        if not code:
            code = """
            No code provided.
            """

        syntax = Syntax(code, language, theme=theme, line_numbers=line_numbers)
        self.console.print(syntax)

    def art(self, message="hammadpy", art="random"):
        """
        Creates an ASCII art styled splash 'logo' in the terminal using Rich Panel.

        Example:
            ```python
            from yosemite.tools.text import Text
            
            text = Text()
            text.art("hammadpy", art="random")
            ```

        Args:
            message (str): The message to display in the splash. Defaults to "hammadpy".
            art (str): The ASCII art style to use. Defaults to "random".
        """
        if art == "random":
            fonts = ["block", "caligraphy", "doh", "dohc", "doom", "epic", "fender", "graffiti", "isometric1", "isometric2", "isometric3", "isometric4", "letters", "alligator", "dotmatrix", "bubble", "bulbhead", "digital", "ivrit", "lean", "mini", "script", "shadow", "slant", "speed", "starwars", "stop", "thin", "3-d", "3x5", "5lineoblique", "acrobatic", "alligator2", "alligator3", "alphabet", "banner", "banner3-D", "banner3", "banner4", "barbwire", "basic", "bell", "big", "bigchief", "binary", "block", "broadway", "bubble", "caligraphy", "doh", "dohc", "doom", "dotmatrix", "drpepper", "epic", "fender", "graffiti", "isometric1", "isometric2", "isometric3", "isometric4", "letters", "alligator", "dotmatrix", "bubble", "bulbhead", "digital", "ivrit", "lean", "mini", "script", "shadow", "slant", "speed", "starwars", "stop", "thin"]
            art = random.choice(fonts)

        art_message = text2art(message, font=art)
        panel = Panel(art_message, expand=False, border_style="bold dark_orange")
        self.console.print(panel)

    def header(self, message, style="bold white", underline="=", overline="="):
        """
        Style and print a header using Rich.

        Args:
            message (str): The message to be styled and printed.
            style (str, optional): The style of the text. Defaults to "bold white".
            underline (str, optional): The character to use for underlining. Defaults to "=".
            overline (str, optional): The character to use for overlining. Defaults to "=".
        """
        self.console.print(message, style=style)
        self.console.print(underline * len(message), style=style)

    def table(self, data: list, title: str = None, title_style: str = "bold yellow", header_style: str = "bold cyan", row_styles: list = None):
        """
        Display a set of data as a table using Rich.

        Args:
            data (list): A list of lists representing the table data.
            title (str, optional): The title of the table. Defaults to None.
            title_style (str, optional): The style of the table title. Defaults to "bold yellow".
            header_style (str, optional): The style of the table header. Defaults to "bold cyan".
            row_styles (list, optional): A list of styles for each row. Defaults to None.
        """
        table = Table(title=title, title_style=title_style, header_style=header_style)

        # Add columns based on the number of elements in the first row
        for _ in data[0]:
            table.add_column()

        # Add rows to the table
        for i, row in enumerate(data):
            if row_styles and i < len(row_styles):
                table.add_row(*row, style=row_styles[i])
            else:
                table.add_row(*row)

        self.console.print(table)

#==============================================================================

if __name__ == "__main__":
    text = Text()
    text.art("hammadpy", art="random")
    text.emoji("thumbs_up", style="bold white")
    text.display("Hello, Rich!", style="bold red", use_box=True)
    text.code("print('Hello, Rich!')", language="python", theme="monokai", line_numbers=True)
    text.header("Hello, Rich!", style="bold red", underline="-", overline="-")

    data = [
        ["Name", "Age", "Location"],
        ["Alice", "25", "New York"],
        ["Bob", "30", "San Francisco"],
        ["Charlie", "35", "Los Angeles"]
    ]

    row_styles = ["bold green", "bold blue", "bold magenta", "bold cyan"]

    text.table(data, title="User Information", title_style="bold yellow", header_style="bold cyan", row_styles=row_styles)


from .cli import CLI
from .dialogs import Dialog
from .frontend import Frontend
from .inputs import Input
from .live_table import LiveTable
from .loaders import Loader
from .logger import Logger
from .preconditions import Preconditions
from .progress_bar import ProgressBar
from .tailwind import Tailwind
from .text import Text


import fire

#==============================================================================

class CLI:
    def __init__(self):
        """
        A lightweight framework for creating CLI apps using Python Fire.

        This class provides a simple and intuitive way to define and run CLI
        commands using Python Fire. It allows you to quickly create powerful
        CLI apps with minimal boilerplate code.

        Attributes:
            command: A decorator to define a CLI command.
            run: A function to run the CLI app.

        Example:
            ```python
            from yosemite.tools.cli import CLI

            @CLI.command()
            def hello(name="World"):
                return f"Hello, {name}!"

            @CLI.command(name="greet")
            def greeting(name="World"):
                return f"Greetings, {name}!"

            if __name__ == "__main__":
                CLI.run()
            ```

            This example defines two CLI commands: `hello` and `greet`. The `hello`
            command takes an optional `name` argument and returns a hello message.
            The `greet` command also takes an optional `name` argument and returns
            a greeting message.

            To run the CLI app, simply call `CLI.run()` at the end of the script.

            ```
            $ python example.py hello
            Hello, World!

            $ python example.py hello --name="Alice"
            Hello, Alice!

            $ python example.py greet
            Greetings, World!

            $ python example.py greet --name="Bob"
            Greetings, Bob!
            ```
        """
        pass

    @staticmethod
    def create(name=None, **kwargs):
        """
        A decorator to define a CLI command.

        This decorator is used to define a function as a CLI command. It wraps
        the function with the `fire.decorators.SetMetadata` decorator from
        Python Fire, which sets the command name and any additional metadata.

        Args:
            name (str): The name of the command. If not provided, the function
                        name is used as the command name.
            **kwargs: Additional keyword arguments to pass to the Fire decorator.

        Returns:
            The decorated function.

        Example:
            ```python
            @CLI.command()
            def hello(name="World"):
                return f"Hello, {name}!"

            @CLI.command(name="greet")
            def greeting(name="World"):
                return f"Greetings, {name}!"
            ```

            In this example, the `hello` function is defined as a CLI command
            with the same name as the function. The `greeting` function is also
            defined as a CLI command, but with the name "greet".
        """
        def decorator(func):
            return fire.decorators.SetMetadata(func, name, **kwargs)
        return decorator

    @staticmethod
    def run(component=None, **kwargs):
        """
        A function to run the CLI app.

        This function runs the CLI app using Python Fire. It takes an optional
        `component` argument, which specifies the component to run the CLI app
        on. If not provided, the calling module is used.

        Args:
            component: The component to run the CLI app on. If not provided,
                       the calling module is used.
            **kwargs: Additional keyword arguments to pass to the Fire CLI.

        Example:
            ```python
            if __name__ == "__main__":
                CLI.run()
            ```

            This example runs the CLI app on the calling module.

            ```python
            def main():
                CLI.run(component=some_module)

            if __name__ == "__main__":
                main()
            ```

            This example runs the CLI app on the specified `some_module` component.
        """
        if component is None:
            fire.Fire(**kwargs)
        else:
            fire.Fire(component, **kwargs)

#==============================================================================

"""
To test this script, try running the following commands:

$ python cli.py hello
Hello, World!

$ python cli.py hello --name="Alice"
Hello, Alice!
"""

if __name__ == "__main__":
    @CLI.create()
    def hello(name="World"):
        """
        A simple command that greets the user.

        Args:
            name (str): The name to greet. Defaults to "World".

        Returns:
            A greeting message.
        """
        return f"Hello, {name}!"

    @CLI.create(name="greet")
    def greeting(name="World"):
        """
        A command that generates a greeting message.

        Args:
            name (str): The name to include in the greeting. Defaults to "World".

        Returns:
            A greeting message.
        """
        return f"Greetings, {name}!"

    CLI.run()


from rich.console import Console
from rich.status import Status
from rich.panel import Panel
from rich import box

# ==============================================================================

class Loader:
    """Displays an animated loading status using the Rich library.

    Attributes:
        message (str): The message to be displayed while loading.
        spinner (str): The spinner style to be used for the loading animation.
        use_box (bool): Whether to wrap the loading message in a use_box.
    """

    def __init__(self, message: str = "Loading...", spinner: str = "dots", use_box: bool = False):
        """
        Initializes the RichLoader with a message, spinner style, and use_box option.

        Example:
            ```python
            from yosemite.tools.loaders import Loader

            loader = RichLoader(message="Processing data", spinner="dots", use_box=True)
            with loader:
                time.sleep(2)
                loader.update("Processing step 1")
                time.sleep(2)
                loader.update("Processing step 2")
                time.sleep(2)
            ```

        Args:
            message (str, optional): The message to be displayed while loading. Defaults to "Loading...".
            spinner (str, optional): The spinner style to be used for the loading animation. Defaults to "dots".
            use_box (bool, optional): Whether to wrap the loading message in a use_box. Defaults to False.
        """
        self.console = Console()
        self.message = message
        self.spinner = spinner
        self.use_box = use_box

    def __enter__(self):
        if self.use_box:
            self.status = Status(Panel(self.message, expand=False, box=box.ROUNDED), spinner=self.spinner)
        else:
            self.status = Status(self.message, spinner=self.spinner)
        self.status.start()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.status.stop()

    def update(self, message: str):
        """Updates the loading message while the loader is running.

        Example:
            ```python
            loader = Loader()
            with loader:
                time.sleep(2)
                loader.update("Processing step 1")
                time.sleep(2)
                loader.update("Processing step 2")
                time.sleep(2)
            ```

        Args:
            message (str): The updated message to be displayed.
        """
        self.status.update(Panel(message, expand=False, box=box.ROUNDED) if self.use_box else message)

# ==============================================================================

if __name__ == "__main__":
    import time
    
    loader = Loader(message="Processing data", spinner="dots", use_box=True)
    with loader:
        time.sleep(2)
        loader.update("Processing step 1")
        time.sleep(2)
        loader.update("Processing step 2")
        time.sleep(2)


from IPython.display import display, HTML
import os

class Tailwind:
    """
    A class for creating and displaying HTML elements with Tailwind CSS, Material-UI styles, and custom fonts in Jupyter notebooks.
    
    Attributes:
        html_template (str): The HTML template used to create the elements.
    """
    def __init__(self):
        self.html_template = """
        <!DOCTYPE html>
        <html>
        <head>
            <script src="https://cdn.tailwindcss.com"></script>
            <script src="https://unpkg.com/@material-ui/core@latest/umd/material-ui.development.js" crossorigin="anonymous"></script>
            <script src="https://unpkg.com/babel-standalone@latest/babel.min.js" crossorigin="anonymous"></script>
            <link rel="stylesheet" href="https://fonts.googleapis.com/css?family=Roboto:300,400,500,700&display=swap" />
            <link href="https://fonts.googleapis.com/css2?family=DM+Sans:wght@400;500;700&family=Lexend:wght@400;500;600;700&family=Lora:wght@400;500;600;700&family=Playfair+Display:wght@400;500;600;700;800;900&family=JetBrains+Mono:wght@400;500;600;700&display=swap" rel="stylesheet">
            <style>
                body {{
                    font-family: 'DM Sans', sans-serif;
                }}
                h1, h2, h3, h4, h5, h6 {{
                    font-family: 'Lexend', sans-serif;
                }}
                p {{
                    font-family: 'Lora', serif;
                }}
                code, pre {{
                    font-family: 'JetBrains Mono', monospace;
                }}
                .playfair {{
                    font-family: 'Playfair Display', serif;
                }}
            </style>
        </head>
        <body>
            <div id="root"></div>
            <script type="text/babel">
                const {{ content }} = MaterialUI;
                ReactDOM.render(
                    <div>
                        {content}
                    </div>,
                    document.getElementById('root')
                );
            </script>
        </body>
        </html>
        """

    def display(self, content: str, className: str = "", component: str = "div", mui: bool = False):
        """
        Display the HTML element with the specified content, component type, Tailwind CSS class, and optionally use Material-UI.

        Args:
            content (str): The content of the HTML element.
            component (str, optional): The HTML component type. Defaults to "div".
            className (str, optional): The Tailwind CSS class name. Defaults to an empty string.
            mui (bool, optional): Whether to use Material-UI components. Defaults to False.
        """
        if mui:
            element = f"<{component} className='{{{{ classes.root }}}} {className}'>{content}</{component}>"
            mui_imports = "{ Button, Typography }"
            html = self.html_template.format(content=element, mui_imports=mui_imports)
        else:
            element = f"<{component} class='{className}'>{content}</{component}>"
            html = self.html_template.format(content=element, mui_imports="")
        
        display(HTML(html))

# Example usage
if __name__ == "__main__":
    tailwind = Tailwind()
    tailwind.display("Hello, Tailwind!", component="h1", className="text-4xl font-bold text-blue-500")
    tailwind.display("Hello, Material-UI!", component="Button", className="bg-blue-500 text-white px-4 py-2 rounded", mui=True)
    tailwind.display("This is a paragraph with Lora font.", component="p", className="text-lg")
    tailwind.display("This is a code snippet with JetBrains Mono font.", component="code", className="text-sm")
    tailwind.display("This is a heading with Playfair Display font.", component="h2", className="text-3xl playfair")


import os
import sys
from typing import Optional, Union, List, Dict, Tuple, Any
import customtkinter
from PIL import Image

class Frontend:
    """
    A lightweight and easy-to-use inline user interface module for creating simple GUIs with CustomTkinter.

    Example usage:
        ```python
        from yosemite.tools.frontend import Frontend

        ui = Frontend()
        ui.create("My App", "600x400")
        ui.set_grid_layout(3, 2)
        name_var = ui.add_textbox("Enter your name", row=1, column=1)
        ui.add_button("Click Me", command=button_click, args=[name_var], row=1, column=0)
        ui.start()

        def button_click(name):
            print(f"Button clicked! Name: {name.get()}")
        ```

    Methods:
        - create(title: str, geometry: str): Create the main application window.
        - set_grid_layout(rows: int, columns: int): Set the grid layout for the main window.
        - add_label(text: str, row: int, column: int, **kwargs): Add a label widget to the specified grid cell.
        - add_button(text: str, command: Optional[callable], args: List[Any], row: int, column: int, **kwargs): Add a button widget to the specified grid cell.
        - add_textbox(placeholder: str, row: int, column: int, **kwargs): Add a textbox widget to the specified grid cell and return the StringVar associated with it.
        - add_checkbox(text: str, command: Optional[callable], args: List[Any], row: int, column: int, **kwargs): Add a checkbox widget to the specified grid cell and return the BooleanVar associated with it.
        - add_radio_button(text: str, value: str, row: int, column: int, **kwargs): Add a radio button widget to the specified grid cell.
        - add_slider(from_: float, to: float, command: Optional[callable], args: List[Any], row: int, column: int, **kwargs): Add a slider widget to the specified grid cell and return the DoubleVar associated with it.
        - add_option_menu(values: List[str], command: Optional[callable], args: List[Any], row: int, column: int, **kwargs): Add an option menu widget to the specified grid cell and return the StringVar associated with it.
        - add_image(path: str, size: Tuple[int, int], row: int, column: int, **kwargs): Add an image widget to the specified grid cell.
        - set_font(font_family: str, size: int): Set the default font for all widgets.
        - start(): Start the main event loop of the application.
    """

    def __init__(self):
        customtkinter.set_appearance_mode("System")
        customtkinter.set_default_color_theme("blue")
        self.app = customtkinter.CTk()
        self.font = customtkinter.CTkFont(family="Arial", size=12)
        self.radio_var = None

    def create(self, title: str = "InlineUI App", geometry: str = "600x400"):
        """
        Create the main application window.

        Args:
            title (str): The title of the window. Default is "InlineUI App".
            geometry (str): The geometry of the window in the format "widthxheight". Default is "600x400".
        """
        self.app.title(title)
        self.app.geometry(geometry)

    def set_grid_layout(self, rows: int = 1, columns: int = 1):
        """
        Set the grid layout for the main window.

        Args:
            rows (int): The number of rows in the grid. Default is 1.
            columns (int): The number of columns in the grid. Default is 1.
        """
        for i in range(rows):
            self.app.grid_rowconfigure(i, weight=1)
        for i in range(columns):
            self.app.grid_columnconfigure(i, weight=1)

    def add_label(self, text: str = "", row: int = 0, column: int = 0, **kwargs):
        """
        Add a label widget to the specified grid cell.

        Args:
            text (str): The text to display in the label. Default is an empty string.
            row (int): The row index of the grid cell. Default is 0.
            column (int): The column index of the grid cell. Default is 0.
            **kwargs: Additional keyword arguments to pass to the CTkLabel constructor.
        """
        label = customtkinter.CTkLabel(self.app, text=text, font=self.font, **kwargs)
        label.grid(row=row, column=column, padx=10, pady=10, sticky="nsew")

    def add_button(self, text: str = "Button", command: Optional[callable] = None, args: List[Any] = None, row: int = 0, column: int = 0, **kwargs):
        """
        Add a button widget to the specified grid cell.

        Args:
            text (str): The text to display on the button. Default is "Button".
            command (Optional[callable]): The function to be called when the button is clicked. Default is None.
            args (List[Any]): The arguments to pass to the command function. Default is None.
            row (int): The row index of the grid cell. Default is 0.
            column (int): The column index of the grid cell. Default is 0.
            **kwargs: Additional keyword arguments to pass to the CTkButton constructor.
        """
        if args is None:
            args = []
        button = customtkinter.CTkButton(self.app, text=text, command=lambda: command(*args), font=self.font, **kwargs)
        button.grid(row=row, column=column, padx=10, pady=10, sticky="nsew")

    def add_textbox(self, placeholder: str = "Enter text", row: int = 0, column: int = 0, **kwargs):
        """
        Add a textbox widget to the specified grid cell.

        Args:
            placeholder (str): The placeholder text for the textbox. Default is "Enter text".
            row (int): The row index of the grid cell. Default is 0.
            column (int): The column index of the grid cell. Default is 0.
            **kwargs: Additional keyword arguments to pass to the CTkEntry constructor.

        Returns:
            customtkinter.StringVar: The StringVar associated with the textbox.
        """
        textvar = customtkinter.StringVar()
        textbox = customtkinter.CTkEntry(self.app, placeholder_text=placeholder, textvariable=textvar, font=self.font, **kwargs)
        textbox.grid(row=row, column=column, padx=10, pady=10, sticky="nsew")
        return textvar

    def add_checkbox(self, text: str = "Checkbox", command: Optional[callable] = None, row: int = 0, column: int = 0, **kwargs):
        """
        Add a checkbox widget to the specified grid cell.

        Args:
            text (str): The text to display next to the checkbox. Default is "Checkbox".
            command (Optional[callable]): The function to be called when the checkbox is clicked. Default is None.
            row (int): The row index of the grid cell. Default is 0.
            column (int): The column index of the grid cell. Default is 0.
            **kwargs: Additional keyword arguments to pass to the CTkCheckBox constructor.

        Returns:
            customtkinter.BooleanVar: The BooleanVar associated with the checkbox.
        """
        checkvar = customtkinter.BooleanVar()
        checkbox = customtkinter.CTkCheckBox(self.app, text=text, variable=checkvar, command=lambda: command(checkvar) if command else None, font=self.font, **kwargs)
        checkbox.grid(row=row, column=column, padx=10, pady=10, sticky="nsew")
        return checkvar

    def add_radio_button(self, text: str = "Radio Button", value: str = "", row: int = 0, column: int = 0, **kwargs):
        """
        Add a radio button widget to the specified grid cell.

        Args:
            text (str): The text to display next to the radio button. Default is "Radio Button".
            value (str): The value to assign to the radio button. Default is an empty string.
            row (int): The row index of the grid cell. Default is 0.
            column (int): The column index of the grid cell. Default is 0.
            **kwargs: Additional keyword arguments to pass to the CTkRadioButton constructor.
        """
        if self.radio_var is None:
            self.radio_var = customtkinter.StringVar(value=value)
        radio_button = customtkinter.CTkRadioButton(self.app, text=text, variable=self.radio_var, value=value, font=self.font, **kwargs)
        radio_button.grid(row=row, column=column, padx=10, pady=10, sticky="nsew")

    def add_slider(self, from_: float = 0, to: float = 100, command: Optional[callable] = None, args: List[Any] = None, row: int = 0, column: int = 0, **kwargs):
        """
        Add a slider widget to the specified grid cell.

        Args:
            from_ (float): The minimum value of the slider. Default is 0.
            to (float): The maximum value of the slider. Default is 100.
            command (Optional[callable]): The function to be called when the slider value changes. Default is None.
            args (List[Any]): The arguments to pass to the command function. Default is None.
            row (int): The row index of the grid cell. Default is 0.
            column (int): The column index of the grid cell. Default is 0.
            **kwargs: Additional keyword arguments to pass to the CTkSlider constructor.

        Returns:
            customtkinter.DoubleVar: The DoubleVar associated with the slider.
        """
        if args is None:
            args = []
        slidervar = customtkinter.DoubleVar()
        slider = customtkinter.CTkSlider(self.app, from_=from_, to=to, variable=slidervar, command=lambda value: command(value, *args), **kwargs)
        slider.grid(row=row, column=column, padx=10, pady=10, sticky="nsew")
        return slidervar

    def add_option_menu(self, values: List[str] = ["Option 1", "Option 2"], command: Optional[callable] = None, args: List[Any] = None, row: int = 0, column: int = 0, **kwargs):
        """
        Add an option menu widget to the specified grid cell.

        Args:
            values (List[str]): The list of options to display in the menu. Default is ["Option 1", "Option 2"].
            command (Optional[callable]): The function to be called when an option is selected. Default is None.
            args (List[Any]): The arguments to pass to the command function. Default is None.
            row (int): The row index of the grid cell. Default is 0.
            column (int): The column index of the grid cell. Default is 0.
            **kwargs: Additional keyword arguments to pass to the CTkOptionMenu constructor.

        Returns:
            customtkinter.StringVar: The StringVar associated with the option menu.
        """
        if args is None:
            args = []
        optionvar = customtkinter.StringVar(value=values[0])
        option_menu = customtkinter.CTkOptionMenu(self.app, values=values, variable=optionvar, command=lambda choice: command(choice, *args), font=self.font, **kwargs)
        option_menu.grid(row=row, column=column, padx=10, pady=10, sticky="nsew")
        return optionvar

    def add_image(self, path: str, size: Tuple[int, int] = (200, 200), row: int = 0, column: int = 0, **kwargs):
        """
        Add an image widget to the specified grid cell.

        Args:
            path (str): The path to the image file.
            size (Tuple[int, int]): The size of the image in the format (width, height). Default is (200, 200).
            row (int): The row index of the grid cell. Default is 0.
            column (int): The column index of the grid cell. Default is 0.
            **kwargs: Additional keyword arguments to pass to the CTkLabel constructor.
        """
        image = customtkinter.CTkImage(Image.open(path), size=size)
        image_label = customtkinter.CTkLabel(self.app, image=image, text="", **kwargs)
        image_label.grid(row=row, column=column, padx=10, pady=10, sticky="nsew")

    def set_font(self, font_family: str = "Arial", size: int = 12):
        """
        Set the default font for all widgets.

        Args:
            font_family (str): The font family name. Default is "Arial".
            size (int): The font size. Default is 12.
        """
        self.font = customtkinter.CTkFont(family=font_family, size=size)

    def start(self):
        """
        Start the main event loop of the application.
        """
        self.app.mainloop()


if __name__ == "__main__":
    ui = Frontend()
    ui.create("My App", "600x400")
    ui.set_grid_layout(5, 2)
    ui.set_font("Helvetica", 14)

    name_var = ui.add_textbox("Enter your name", row=1, column=1)

    def button_click(name):
        print(f"Button clicked! Name: {name.get()}")

    ui.add_label("Welcome to My App", row=0, column=0)
    ui.add_button("Click Me", command=button_click, args=[name_var], row=1, column=0)

    def checkbox_click(checkvar):
        print(f"Checkbox clicked! Value: {checkvar.get()}")

    check_var = ui.add_checkbox("I agree to the terms and conditions", command=checkbox_click, row=2, column=0)

    ui.add_radio_button("Option 1", value="option1", row=2, column=1)
    ui.add_radio_button("Option 2", value="option2", row=3, column=1)

    def slider_changed(value):
        print(f"Slider value changed: {value}")

    slider_var = ui.add_slider(0, 100, command=slider_changed, row=3, column=0)

    def option_selected(choice):
        print(f"Option selected: {choice}")

    option_var = ui.add_option_menu(["Option 1", "Option 2", "Option 3"], command=option_selected, row=4, column=0)

    ui.start() 


from monterey.tools.text import Text
from datetime import datetime
import logging
import os
from typing import Optional

class Logger:
    """
    A custom logger class for logging messages with different severity levels.

    Attributes:
        logger: A logging.Logger instance for logging messages.
        log_dir: The directory where log files will be stored.
        text: A RichText instance for styled output.
        verbose: A boolean indicating whether to display verbose output.
    """

    def __init__(self, name : str = "logger", log_dir : str ="logs", verbose : bool = False):
        """
        Initialize the Logger instance.

        Example:
            ```python
            from yosemite.tools.logger import Logger

            logger = Logger("test", verbose=True)
            logger.debug("This is a debug message.")
            ```

        Args:
            name (str): The name of the logger.
            log_dir (str, optional): The directory where log files will be stored. Defaults to "logs".
            verbose (bool, optional): Whether to display verbose output. Defaults to False.
        """
        self.logger = logging.getLogger(name)
        self.logger.setLevel(logging.DEBUG)

        self.log_dir = log_dir
        os.makedirs(self.log_dir, exist_ok=True)

        log_file = f"{self.log_dir}/{name}_{datetime.now().strftime('%Y%m%d_%H%M%S')}.log"
        file_handler = logging.FileHandler(log_file)
        file_handler.setLevel(logging.DEBUG)

        formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
        file_handler.setFormatter(formatter)

        self.logger.addHandler(file_handler)

        self.text = Text()
        self.verbose = verbose

    def get_datetime(self):
        """
        Get the current date and time.

        Returns:
            str: The current date and time.
        """
        time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        time = str(f"[{time}]")
        return time

    def debug(self, message : str = None, module : Optional[str] = None):
        """
        Log a debug message.

        Args:
            message (str): The message to be logged.
        """
        time = self.get_datetime()
        self.logger.debug(message)
        if module:
            if self.verbose:
                self.text.display(f"[bold white]{self.logger.name}[/bold white] - [not bold white]{module}[/not bold white] | [grey not bold]{time}[grey not bold] [bold blue]DEBUG[/bold blue] - [dim grey not bold]{message}[/dim grey not bold]")
            else:
                pass
        else:
            if self.verbose:
                self.text.display(f" [bold white]{self.logger.name}[/bold white] | [grey not bold]{time}[grey not bold] [bold blue]DEBUG[/bold blue] - [dim grey not bold]{message}[/dim grey not bold]")
            else:
                pass

    def info(self, message : str = None, module : Optional[str] = None):
        """
        Log an status message.

        Args:
            message (str): The message to be logged.
        """
        time = self.get_datetime()
        self.logger.info(message)
        if module:
            if self.verbose:
                self.text.display(f"[bold white]{self.logger.name}[/bold white] - [not bold white]{module}[/not bold white] | [grey not bold]{time}[grey not bold] [bold green]INFO[/bold green] - [dim grey not bold]{message}[/dim grey not bold]")
            else:
                pass
        else:
            if self.verbose:
                self.text.display(f"[bold white]{self.logger.name}[/bold white] | [grey not bold]{time}[grey not bold] [bold green]INFO[/bold green] - [dim grey not bold]{message}[/dim grey not bold]")

    def warning(self, message : str = None, module : Optional[str] = None):
        """
        Log a warning message.

        Args:
            message (str): The message to be logged.
        """
        time = self.get_datetime()
        self.logger.warning(message)
        if module:
            if self.verbose:
                self.text.display(f"[bold white]{self.logger.name}[/bold white] - [not bold white]{module}[/not bold white] | [grey not bold]{time}[grey not bold] [bold yellow]WARNING[/bold yellow] - [dim grey not bold]{message}[/dim grey not bold]")
            else:
                pass
        else:
            if self.verbose:
                self.text.display(f"[bold white]{self.logger.name}[/bold white] | [grey not bold]{time}[grey not bold] [bold yellow]WARNING[/bold yellow] - [dim grey not bold]{message}[/dim grey not bold]")
            else:
                pass

    def error(self, message : str = None, module : Optional[str] = None):
        """
        Log an error message.

        Args:
            message (str): The message to be logged.
        """
        time = self.get_datetime()
        self.logger.error(message)
        if module:
            if self.verbose:
                self.text.display(f"[bold white]{self.logger.name}[/bold white] - [not bold white]{module}[/not bold white] | [grey not bold]{time}[grey not bold] [bold red]ERROR[/bold red] - [dim grey not bold]{message}[/dim grey not bold]")
            else:
                pass
        else:
            if self.verbose:
                self.text.display(f"[bold white]{self.logger.name}[/bold white] | [grey not bold]{time}[grey not bold] [bold red]ERROR[/bold red] - [dim grey not bold]{message}[/dim grey not bold]")
            else:
                pass

    def critical(self, message : str = None, module : Optional[str] = None):
        """
        Log a critical message.

        Args:
            message (str): The message to be logged.
        """
        time = self.get_datetime()
        self.logger.critical(message)
        if module:
            self.text.display(f"[bold white]{self.logger.name}[/bold white] - [not bold white]{module}[/not bold white] | [grey not bold]{time}[grey not bold] [bold dark_red]CRITICAL[/bold dark_red] - [dim grey not bold]{message}[/dim grey not bold]")
        else:
            self.text.display(f"[bold white]{self.logger.name}[/bold white] | [grey not bold]{time}[grey not bold] [bold dark_red]CRITICAL[/bold dark_red] - [dim grey not bold]{message}[/dim grey not bold]")


if __name__ == "__main__":
    logger = Logger("test", verbose=True)
    logger.debug("This is a debug message.")
    logger.info("This is an info message.", module="test")
    logger.warning("This is a warning message.")
    logger.error("This is an error message.", module="test")
    logger.critical("This is a critical message.")
    pass


from rich.console import Console
from rich.live import Live
from rich.table import Table

# ==============================================================================

class LiveTable:
    """Displays a live updating table using the Rich library."""

    def __init__(self):
        """
        Initializes the RichLiveTable.

        Example:
            ```python
            from yosemite.tools.live_table import LiveTable

            table = LiveTable()
            with table:
                for i in range(10):
                    table.add_row(f"Row {i}", str(random.randint(0, 100)))
                    time.sleep(1)
            ```
        """
        self.console = Console()
        self.table = Table(show_header=True, header_style="bold magenta")
        self.table.add_column("Item", style="dim")
        self.table.add_column("Value")

    def __enter__(self):
        self.live = Live(self.table, console=self.console)
        self.live.start()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.live.stop()

    def add_row(self, item: str, value: str):
        """Adds a row to the live updating table.

        Args:
            item (str): The item name.
            value (str): The item value.
        """
        self.table.add_row(item, value)

# ==============================================================================

if __name__ == "__main__":
    import random
    import time

    table = LiveTable()
    with table:
        for i in range(10):
            table.add_row(f"Row {i}", str(random.randint(0, 100)))
            time.sleep(1)